clear all; close all;

% load images
count = 5;
im_names = cell(count, 1);
for k = 1:count
    im_names{k} = strcat('data/hanoi_mask_', num2str(k), '.png');
end

% read images into lightfield focal stack
[m, n, c] = size(imread(im_names{1}));
stack = zeros([m, n, count]);
for k = 1:count 
    im = rgb2gray(im2double(imread(im_names{k})));
    im(im == 0) = NaN;
    stack(:,:,k) = im;  
end

% compute median image
med_filt = nanmedian(stack, 3);

imshow(med_filt);

% compute confidence, using proportion of NaNs and similarity
Dm = (count - sum(isnan(med_filt), 3) )./count;
Prm_diff = (bsxfun(@minus, stack, med_filt)).^2;
Prm = 1 - nansum(Prm_diff, 3) ./ (count * max(stack.^2, [], 3) ./ 2);
confidence = Dm .* Prm;

imshow(confidence);

r = 4; % window size
w = 7; % search radius
sigma = 0.25; % sigma value
out = stack; % output
for k = 1:count 
    fprintf(1,'Layer: %d / %d  \nRow:        ', k, count);
    for ky = 1+r:m-r
        fprintf(1,'\b\b\b\b\b\b\b\b %d / %d', ky, m-r);
        for kx = 1+r:n-r
            if (isnan(stack(ky, kx, k)))
                % grab current patch
                currentWindow = stack(  ky-r:ky+r, kx-r:kx+r, k);
                normalizationFactor = zeros([1 1 size(med_filt,3)]);

                % search median image for pixel with most similar neighbors
                currentPixel = 0;
                for j = ky-w:ky+w
                    for i = kx-w:kx+w
                        if j>=1+r && j<=size(med_filt,1)-r && i>=1+r && i<=size(med_filt,2)-r && j ~= ky && i ~=kx
                            neighborhood = med_filt(j-r:j+r, i-r:i+r);
                            
                            % if similar neighborhood found, set to that
                            % pixel
                            if (~isnan(med_filt(j, i)))
                                diff = currentWindow - neighborhood;
                                dist = diff .* diff ;
                                weight = exp(-nansum(dist(:))/sigma^2);
                                currentPixel = currentPixel + weight .* med_filt(j, i);
                                normalizationFactor = normalizationFactor + weight;
                            end
                        end
                    end
                end
                
                % write final pixel color to output
                out(ky,kx,k) = currentPixel ./ normalizationFactor;
            end
        end
    end
    fprintf('\n');
    
    figure;
    imshow(out(:,:,k));
    imwrite(out(:,:,k), ['output/median_stack_' num2str(k) '.png']);
end
